@@ -273,35 +273,6 @@ class BaseAuthTokenMiddlewareTest(testtools.TestCase):
                                   httpretty.core.HTTPrettyRequestEmpty)
 
 
-class MultiStepAuthTokenMiddlewareTest(BaseAuthTokenMiddlewareTest,
-                                       testresources.ResourcedTestCase):
-
-    resources = [('examples', client_fixtures.EXAMPLES_RESOURCE)]
-
-    @httpretty.activate
-    def test_fetch_revocation_list_with_expire(self):
-        self.set_middleware()
-
-        # Get a token, then try to retrieve revocation list and get a 401.
-        # Get a new token, try to retrieve revocation list and return 200.
-        httpretty.register_uri(httpretty.POST, "%s/v2.0/tokens" % BASE_URI,
-                               body=FAKE_ADMIN_TOKEN)
-
-        responses = [httpretty.Response(body='', status=401),
-                     httpretty.Response(
-                         body=self.examples.SIGNED_REVOCATION_LIST)]
-
-        httpretty.register_uri(httpretty.GET,
-                               "%s/v2.0/tokens/revoked" % BASE_URI,
-                               responses=responses)
-
-        fetched = jsonutils.loads(self.middleware._fetch_revocation_list())
-        self.assertEqual(fetched, self.examples.REVOCATION_LIST)
-
-        # Check that 4 requests have been made
-        self.assertEqual(len(httpretty.httpretty.latest_requests), 4)
-
-
 class DiabloAuthTokenMiddlewareTest(BaseAuthTokenMiddlewareTest,
                                     testresources.ResourcedTestCase):
 
@@ -671,6 +642,12 @@ class CommonAuthTokenMiddlewareTest(object):
         revoked_form = cms.cms_hash_token(token)
         self._test_cache_revoked(token, revoked_form)
 
+    def test_cached_revoked_pkiz(self):
+        # When the PKIZ token is cached and revoked, 401 is returned.
+        token = self.token_dict['signed_token_scoped_pkiz']
+        revoked_form = cms.cms_hash_token(token)
+        self._test_cache_revoked(token, revoked_form)
+
     def test_revoked_token_receives_401_md5_secondary(self):
         # When hash_algorithms has 'md5' as the secondary hash and the
         # revocation list contains the md5 hash for a token, that token is
@@ -684,7 +661,7 @@ class CommonAuthTokenMiddlewareTest(object):
         self.middleware(req.environ, self.start_fake_response)
         self.assertEqual(self.response_status, 401)
 
-    def test_revoked_hashed_pki_token(self):
+    def _test_revoked_hashed_token(self, token_name):
         # If hash_algorithms is set as ['sha256', 'md5'],
         # and check_revocations_for_cached is True,
         # and a token is in the cache because it was successfully validated
@@ -695,7 +672,7 @@ class CommonAuthTokenMiddlewareTest(object):
         self.conf['check_revocations_for_cached'] = True
         self.set_middleware()
 
-        token = self.token_dict['signed_token_scoped']
+        token = self.token_dict[token_name]
 
         # Put the token in the revocation list.
         token_hashed = cms.cms_hash_token(token)
@@ -709,13 +686,19 @@ class CommonAuthTokenMiddlewareTest(object):
         self.middleware(req.environ, self.start_fake_response)
         self.assertEqual(200, self.response_status)
 
-        # This time use the PKI token
+        # This time use the PKI(Z) token
         req.headers['X-Auth-Token'] = token
         self.middleware(req.environ, self.start_fake_response)
 
         # Should find the token in the cache and revocation list.
         self.assertEqual(401, self.response_status)
 
+    def test_revoked_hashed_pki_token(self):
+        self._test_revoked_hashed_token('signed_token_scoped')
+
+    def test_revoked_hashed_pkiz_token(self):
+        self._test_revoked_hashed_token('signed_token_scoped_pkiz')
+
     def get_revocation_list_json(self, token_ids=None, mode=None):
         if token_ids is None:
             key = 'revoked_token_hash' + (('_' + mode) if mode else '')
@@ -1450,7 +1433,8 @@ class v2AuthTokenMiddlewareTest(BaseAuthTokenMiddlewareTest,
                       self.examples.UUID_TOKEN_BIND,
                       self.examples.UUID_TOKEN_UNKNOWN_BIND,
                       self.examples.UUID_TOKEN_NO_SERVICE_CATALOG,
-                      self.examples.SIGNED_TOKEN_SCOPED_KEY,):
+                      self.examples.SIGNED_TOKEN_SCOPED_KEY,
+                      self.examples.SIGNED_TOKEN_SCOPED_PKIZ_KEY,):
             httpretty.register_uri(httpretty.GET,
                                    "%s/v2.0/tokens/%s" % (BASE_URI, token),
                                    body=
